"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMarker = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = require("react");
var mark_1 = require("./mark");
var keyCounter = 0;
/**
 * Creates a React component class to mark content based on rules.
 *
 * The component takes its children and marks them using the rules of the
 * parsers that are provided.
 *
 * @param parsers A list of Parser systems. A Parser must
 * define a `rule` and a `tag`. The `rule` can be either a string to match
 * terms or a RegExp. Note that a RegExp must have parentheses surrounding
 * your pattern, otherwise matches won't be captured. The `tag` is a function
 * that accepts a string (the matched term or pattern) and returns a React
 * element that will replace the match in the output. Parsers can also pass a
 * `matchIndex` parameter, a number that will be used when using a RegExp to
 * chose which match to pass to the `tag` function.
 *
 * @param wrapTag If defined, wraps each tag with a common wrapper function.
 * The default wrapper returns a clone of the element returned by the tag function,
 * but makes sure that it has a `key` attribute.
 *
 * @returns A functional component that applies `parsers` to its `children`.
 */
var createMarker = function (parsers, wrapTag) {
    return function (_a) {
        var children = _a.children;
        if (!children) {
            return null;
        }
        wrapTag !== null && wrapTag !== void 0 ? wrapTag : (wrapTag = function (tag) { return function (x) {
            return (0, react_1.cloneElement)(tag(x), { key: ++keyCounter });
        }; });
        var res = Array.isArray(children) ? children : [children];
        for (var _i = 0, parsers_1 = parsers; _i < parsers_1.length; _i++) {
            var parser = parsers_1[_i];
            var tag = wrapTag(parser.tag);
            res = (0, mark_1.mark)(res, parser.rule, tag, parser.matchIndex);
        }
        return (0, jsx_runtime_1.jsx)(jsx_runtime_1.Fragment, { children: res });
    };
};
exports.createMarker = createMarker;
